from typing import Any

from ..objects.client import Client
from .base import ResourceGetMixin, ResourceListMixin

__all__ = [
    "Clients",
]


class Clients(ResourceListMixin, ResourceGetMixin):
    """
    Resource handler for the `/clients` endpoint.

    Retrieve a list of Mollie merchants connected to your partner account (only for Mollie partners).
    """

    RESOURCE_ID_PREFIX: str = "org_"

    def get_resource_object(self, result: dict) -> Client:
        return Client(result, self.client)

    def get(self, resource_id: str, **params: Any) -> Client:
        """Retrieve a single client, linked to your partner account, by its ID."""
        self.validate_resource_id(resource_id, "client ID")
        return super().get(resource_id, **params)



from typing import TYPE_CHECKING, Any

from ..objects.chargeback import Chargeback
from ..objects.list import ObjectList
from .base import ResourceBase, ResourceGetMixin, ResourceListMixin

if TYPE_CHECKING:
    from ..client import Client
    from ..objects.payment import Payment
    from ..objects.profile import Profile
    from ..objects.settlement import Settlement

__all__ = [
    "Chargebacks",
    "PaymentChargebacks",
    "ProfileChargebacks",
    "SettlementChargebacks",
]


class ChargebacksBase(ResourceBase):
    RESOURCE_ID_PREFIX: str = "chb_"

    def get_resource_object(self, result: dict) -> Chargeback:
        return Chargeback(result, self.client)


class Chargebacks(ChargebacksBase, ResourceListMixin):
    """Resource handler for the `/chargebacks` endpoint."""

    pass


class PaymentChargebacks(ChargebacksBase, ResourceGetMixin, ResourceListMixin):
    """Resource handler for the `/payments/:payment_id:/chargebacks` endpoint."""

    _payment: "Payment"

    def __init__(self, client: "Client", payment: "Payment") -> None:
        self._payment = payment
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"payments/{self._payment.id}/chargebacks"

    def get(self, resource_id: str, **params: Any) -> Chargeback:
        self.validate_resource_id(resource_id, "chargeback ID")
        return super().get(resource_id, **params)


class SettlementChargebacks(ChargebacksBase, ResourceListMixin):
    """Resource handler for the `/settlements/:settlement_id:/chargebacks` endpoint."""

    _settlement: "Settlement"

    def __init__(self, client: "Client", settlement: "Settlement") -> None:
        self._settlement = settlement
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"settlements/{self._settlement.id}/chargebacks"


class ProfileChargebacks(ChargebacksBase):
    """
    Resource handler for the `/chargebacks?profileId=:profile_id:` endpoint.

    This is separate from the `Chargebacks` resource handler to make it easier to inject the profileId.
    """

    _profile: "Profile"

    def __init__(self, client: "Client", profile: "Profile") -> None:
        self._profile = profile
        super().__init__(client)

    def list(self, **params: Any) -> ObjectList:
        # Set the profileId in the query params
        params.update({"profileId": self._profile.id})
        return Chargebacks(self.client).list(**params)



from typing import TYPE_CHECKING, Any, Dict, Optional

from ..error import DataConsistencyError
from ..objects.list import ObjectList
from ..objects.order_line import OrderLine
from .base import ResourceBase

if TYPE_CHECKING:
    from ..client import Client
    from ..objects.order import Order

__all__ = [
    "OrderLines",
]


class OrderLines(ResourceBase):
    """
    Resource handler for the `/orders/:order_id:/lines` endpoint.

    This class provides various generic methods such as .delete(), .update() and
    .list(), but since the API interface for these methods is completely different
    from other endpoints, all methods don't use the generic methods but have custom
    implementations in this class.
    """

    RESOURCE_ID_PREFIX: str = "odl_"

    _order: "Order"

    def __init__(self, client: "Client", order: "Order") -> None:
        self._order = order
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"orders/{self._order.id}/lines"

    def get_resource_object(self, result: dict) -> OrderLine:
        return OrderLine(result, self.client)

    def delete_lines(self, data: Optional[Dict[str, Any]] = None, **params: Any) -> dict:
        """
        Cancel multiple orderlines.

        Orderlines are (partially) canceled by sending a list of lines with id's and
        optionally altered properties to the API. See the Mollie docs for details.

            data = {
                "lines":
                    [
                        {
                            "id": 'odl_dgtxyl',
                            "quantity": 1,
                        }
                    ]
                }
            order.lines.delete_lines(data)

        To cancel all lines, simply omit the data parameter.

            order.lines.delete_lines()

        Note: since this differs significantly from a regular .delete(), the method
        has a different name.
        """
        if data is None:
            data = {"lines": []}

        path = self.get_resource_path()
        return self.perform_api_call(self.REST_DELETE, path, data=data, params=params)

    def delete(self, order_line_id: str, **params: Any) -> dict:
        """
        Cancel a single orderline.

        This method provides the regular .delete() interface on top of .delete_lines().
        """
        self.validate_resource_id(order_line_id, "orderline ID")
        data = {
            "lines": [
                {"id": order_line_id},
            ],
        }
        return self.delete_lines(data, **params)

    def update(self, order_line_id: str, data: Optional[Dict[str, Any]] = None, **params: Any) -> OrderLine:
        """
        Custom handling for updating orderlines.

        We are manipulating an orderline here, but the API returns the full order payload.
        To be more consistent, we return the updated orderline object in stead.

        If you wish to retrieve the order object, you can do so by using the order_id property of the orderline.
        """
        resource_path = self.get_resource_path()
        path = f"{resource_path}/{order_line_id}"
        result = self.perform_api_call(self.REST_UPDATE, path, data=data, params=params)

        for line in result["lines"]:
            if line["id"] == order_line_id:
                return self.get_resource_object(line)

        raise DataConsistencyError(f"OrderLine with id '{order_line_id}' not found in response.")

    def list(self, **params: Any) -> ObjectList:
        """Return the orderline data from the related order."""
        lines = self._order._get_property("lines") or []
        data = {
            "_embedded": {
                "lines": lines,
            },
            "count": len(lines),
        }
        return ObjectList(data, OrderLine, self.client)



import re

from .base import ObjectBase


class Chargeback(ObjectBase):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources import Chargebacks

        return Chargebacks(client)

    @property
    def id(self):
        return self._get_property("id")

    @property
    def amount(self):
        return self._get_property("amount")

    @property
    def settlement_amount(self):
        return self._get_property("settlementAmount")

    @property
    def created_at(self):
        return self._get_property("createdAt")

    @property
    def reason(self):
        return self._get_property("reason")

    @property
    def reversed_at(self):
        return self._get_property("reversedAt")

    @property
    def payment_id(self):
        return self._get_property("paymentId")

    @property
    def settlement_id(self):
        """
        Return the settlement ID.

        It is extracted from the settlement link, since the id is not available as a real property.
        """
        url = self._get_link("settlement")
        if not url:
            return None

        match = re.findall(r"/settlements/(stl_\w+)$", url)
        if match:
            return match[0]

    def get_payment(self):
        """Return the Payment object related to this chargeback."""
        # TODO Use the embedded payment data, if available.
        return self.client.payments.get(self.payment_id)

    def get_settlement(self):
        """Return the Settlement object related to this chargeback, if available."""
        settlement_id = self.settlement_id
        if settlement_id is not None:
            return self.client.settlements.get(settlement_id)



from .base import ObjectBase


class UnknownObject(ObjectBase):
    """Mock object for empty lists."""

    @classmethod
    def get_object_name(cls):
        return "unknown"


class ObjectList(ObjectBase):
    current = None

    def __init__(self, result, object_type, client=None):
        # If an empty dataset was injected, we mock the structure that the remainder of the clas expects.
        # TODO: it would be better if the ObjectList was initiated with a list of results, rather than with
        # the full datastructure as it is now, so we can remove all this mucking around with fake data,
        # mocked result objects, and loads of lengthy accessor workarounds everywhere in the ObjectList.
        if result == {}:
            result = {"_embedded": {"unknown": []}, "count": 0}
            object_type = UnknownObject

        super().__init__(result, client)
        self.object_type = object_type

    def __len__(self):
        """Return the count field."""
        return self.count

    def __iter__(self):
        """Implement iterator interface."""
        self.current = None
        return self

    def __next__(self):
        """Implement iterator interface."""
        if self.current is None:
            self.current = 0
        else:
            self.current += 1
        try:
            item = self["_embedded"][self.object_type.get_object_name()][self.current]
            return self.object_type(item, self.client)
        except IndexError:
            self.current = None
            raise StopIteration

    def __getitem__(self, key):
        """Implement Sequence interface."""
        object_name = self.object_type.get_object_name()
        if isinstance(key, int):
            # Return an index-based search from the "_embedded" dataset
            item = self["_embedded"][object_name][key]
            return self.object_type(item, self.client)

        if isinstance(key, slice):
            _start = key.start or 0
            _stop = key.stop or self["count"]
            _step = key.step or 1
            sliced_data = [self["_embedded"][object_name][x] for x in range(_start, _stop, _step)]
            # Now we mock a result based on the sliced data
            sliced_result = {
                "_embedded": {
                    object_name: sliced_data,
                },
                "count": len(sliced_data),
            }
            return ObjectList(sliced_result, self.object_type, self.client)

        return super().__getitem__(key)

    @property
    def count(self):
        if "count" not in self:
            return None
        return int(self["count"])

    def has_next(self):
        """Return True if the ObjectList contains an url for the next set."""
        return self._get_link("next") is not None

    def has_previous(self):
        """Return True if the ObjectList contains an url for the previous set."""
        return self._get_link("previous") is not None

    def get_next(self):
        """Return the next set of objects in an ObjectList."""
        url = self._get_link("next")
        resource = self.object_type.get_resource_class(self.client)
        resp = resource.perform_api_call(resource.REST_READ, url)
        return ObjectList(resp, self.object_type, self.client)

    def get_previous(self):
        """Return the previous set of objects in an ObjectList."""
        url = self._get_link("previous")
        resource = self.object_type.get_resource_class(self.client)
        resp = resource.perform_api_call(resource.REST_READ, url)
        return ObjectList(resp, self.object_type, self.client)



from typing import TYPE_CHECKING, Any, Dict, Optional

from ..error import IdentifierError, ResponseError, ResponseHandlingError
from ..objects.list import ObjectList

if TYPE_CHECKING:
    from ..client import Client


class ResourceBase:
    DEFAULT_LIMIT = 10

    REST_CREATE: str = "POST"
    REST_READ: str = "GET"
    REST_LIST: str = "GET"
    REST_UPDATE: str = "PATCH"
    REST_DELETE: str = "DELETE"

    RESOURCE_ID_PREFIX: str = ""

    def __init__(self, client: "Client") -> None:
        self.client = client

    def get_resource_object(self, result: dict) -> Any:
        """
        Return an instantiated result class for this resource. Should be overriden by a subclass.

        :param result: The API response that the object should hold.
        :type result: dict
        """
        raise NotImplementedError()  # pragma: no cover

    def get_resource_path(self) -> str:
        """Return the base URL path in the API for this resource."""
        return self.__class__.__name__.lower()

    def perform_api_call(
        self,
        http_method: str,
        path: str,
        data: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        resp = self.client.perform_http_call(http_method, path, data, params)
        if "application/hal+json" in resp.headers.get("Content-Type", ""):
            # set the content type according to the media type definition
            resp.encoding = "utf-8"
        try:
            result = resp.json() if resp.status_code != 204 else {}
        except Exception:
            raise ResponseHandlingError(
                f"Unable to decode Mollie API response (status code: {resp.status_code}): '{resp.text}'."
            )
        if resp.status_code < 200 or resp.status_code > 299:
            if "status" in result and (result["status"] < 200 or result["status"] > 299):
                # the factory will return the appropriate ResponseError subclass based on the result
                raise ResponseError.factory(result)
            else:
                raise ResponseHandlingError(
                    "Received HTTP error from Mollie API, but no status in payload "
                    f"(status code: {resp.status_code}): '{resp.text}'."
                )
        return result

    @classmethod
    def validate_resource_id(cls, resource_id: str, name: str = "Identifier", message: str = "") -> None:
        """Generic identifier validation."""
        if not message:
            message = f"Invalid {name} '{resource_id}', it should start with '{cls.RESOURCE_ID_PREFIX}'."

        if not resource_id or not str(resource_id).startswith(cls.RESOURCE_ID_PREFIX):
            raise IdentifierError(message)


class ResourceCreateMixin(ResourceBase):
    def create(self, data: Optional[Dict[str, Any]] = None, **params: Any) -> Any:
        path = self.get_resource_path()
        result = self.perform_api_call(self.REST_CREATE, path, data, params)
        return self.get_resource_object(result)


class ResourceGetMixin(ResourceBase):
    def get(self, resource_id: str, **params: Any) -> Any:
        resource_path = self.get_resource_path()
        path = f"{resource_path}/{resource_id}"
        result = self.perform_api_call(self.REST_READ, path, params=params)
        return self.get_resource_object(result)

    def from_url(self, url: str, params: Optional[Dict[str, Any]] = None) -> Any:
        """Utility method to return an object from a full URL (such as from _links).

        This method always does a GET request and returns a single Object.
        """
        result = self.perform_api_call(self.REST_READ, url, params=params)
        return self.get_resource_object(result)


class ResourceListMixin(ResourceBase):
    def list(self, **params: Any) -> ObjectList:
        path = self.get_resource_path()
        result = self.perform_api_call(self.REST_LIST, path, params=params)
        return ObjectList(result, self.get_resource_object({}).__class__, self.client)


class ResourceUpdateMixin(ResourceBase):
    def update(self, resource_id: str, data: Optional[Dict[str, Any]] = None, **params: Any) -> Any:
        resource_path = self.get_resource_path()
        path = f"{resource_path}/{resource_id}"
        result = self.perform_api_call(self.REST_UPDATE, path, data, params)
        return self.get_resource_object(result)


class ResourceDeleteMixin(ResourceBase):
    def delete(self, resource_id: str, **params: Any) -> Any:
        resource_path = self.get_resource_path()
        path = f"{resource_path}/{resource_id}"
        return self.perform_api_call(self.REST_DELETE, path, params=params)



from .base import ObjectBase


class Client(ObjectBase):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources import Clients

        return Clients(client)

    # Documented properties

    @property
    def id(self):
        return self._get_property("id")

    @property
    def resource(self):
        return self._get_property("resource")

    @property
    def organisation_created_at(self):
        return self._get_property("organizationCreatedAt")

    # documented _links

    def get_organization(self):
        """Return the client’s organization. Only available when the include could have been used."""
        url = self._get_link("organization")
        if url:
            return self.client.organizations.from_url(url)
        else:
            return None

    def get_onboarding(self):
        """Return the client’s onboarding status. Only available when the include could have been used."""
        url = self._get_link("onboarding")
        if url:
            return self.client.onboarding.from_url(url)
        else:
            return None



from typing import TYPE_CHECKING, Any

from ..objects.capture import Capture
from .base import ResourceBase, ResourceGetMixin, ResourceListMixin

if TYPE_CHECKING:
    from ..client import Client
    from ..objects.payment import Payment
    from ..objects.settlement import Settlement

__all__ = [
    "PaymentCaptures",
    "SettlementCaptures",
]


class CapturesBase(ResourceBase):
    RESOURCE_ID_PREFIX: str = "cpt_"

    def get_resource_object(self, result: dict) -> Capture:
        return Capture(result, self.client)


class PaymentCaptures(CapturesBase, ResourceGetMixin, ResourceListMixin):
    """Resource handler for the `/payments/:payment_id:/captures` endpoint."""

    _payment: "Payment"

    def __init__(self, client: "Client", payment: "Payment") -> None:
        self._payment = payment
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"payments/{self._payment.id}/captures"

    def get(self, resource_id: str, **params: Any) -> Capture:
        self.validate_resource_id(resource_id, "capture ID")
        return super().get(resource_id, **params)


class SettlementCaptures(CapturesBase, ResourceListMixin):
    """Resource handler for the `/settlements/:settlement_id:/captures` endpoint."""

    _settlement: "Settlement"

    def __init__(self, client: "Client", settlement: "Settlement") -> None:
        self._settlement = settlement
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"settlements/{self._settlement.id}/captures"



import re
from typing import Any, Pattern

from ..error import IdentifierError
from ..objects.settlement import Settlement
from .base import ResourceGetMixin, ResourceListMixin


class Settlements(ResourceGetMixin, ResourceListMixin):
    """Resource handler for the `/settlements` endpoint."""

    RESOURCE_ID_PREFIX: str = "stl_"

    # According to Mollie, the bank reference is formatted as:
    # - The Mollie customer ID, 4 to 7 digits.
    # - The year and month, 4 digits
    # - The sequence number of the settlement in that month, 2 digits
    # The components are separated by a dot.
    BANK_REFERENCE_REGEX: Pattern[str] = re.compile(r"^\d{4,7}\.\d{4}\.\d{2}$", re.ASCII)

    def get_resource_object(self, result: dict) -> Settlement:
        return Settlement(result, self.client)

    @classmethod
    def validate_resource_id(cls, resource_id: str, name: str = "", message: str = "") -> None:
        """
        Validate the reference id to a settlement.

        Valid references for settlements are:
        - The string "next"
        - The string "open"
        - A settlement id, starting with "stl_"
        - A bank reference
        """
        exc_message = (
            f"Invalid settlement ID '{resource_id}', it should start with '{cls.RESOURCE_ID_PREFIX}', "
            "be 'next' or 'open', or contain a valid bank reference."
        )

        if resource_id in ["next", "open"]:
            return

        elif cls.BANK_REFERENCE_REGEX.match(str(resource_id)):
            return

        else:
            try:
                super().validate_resource_id(resource_id)
            except IdentifierError:
                raise IdentifierError(exc_message)

    def get(self, resource_id: str, **params: Any) -> Settlement:
        self.validate_resource_id(resource_id)
        return super().get(resource_id, **params)



from typing import TYPE_CHECKING, Any, Dict, Optional

from ..objects.list import ObjectList
from ..objects.order import Order
from ..objects.payment import Payment
from ..objects.profile import Profile
from ..objects.refund import Refund
from .base import ResourceBase, ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin

if TYPE_CHECKING:
    from ..client import Client
    from ..objects.settlement import Settlement

__all__ = [
    "OrderRefunds",
    "PaymentRefunds",
    "ProfileRefunds",
    "Refunds",
    "SettlementRefunds",
]


class RefundsBase(ResourceBase):
    RESOURCE_ID_PREFIX: str = "re_"

    def get_resource_object(self, result: dict) -> Refund:
        return Refund(result, self.client)


class Refunds(RefundsBase, ResourceListMixin):
    """Resource handler for the `/refunds` endpoint."""

    pass


class PaymentRefunds(RefundsBase, ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin):
    """Resource handler for the `/payments/:payment_id:/refunds` endpoint."""

    _payment: Payment

    def __init__(self, client: "Client", payment: Payment) -> None:
        self._payment = payment
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"payments/{self._payment.id}/refunds"

    def get(self, resource_id: str, **params: Any) -> Refund:
        self.validate_resource_id(resource_id, "Refund ID")
        return super().get(resource_id, **params)

    def delete(self, resource_id: str, **params: Any) -> dict:
        self.validate_resource_id(resource_id, "Refund ID")
        return super().delete(resource_id, **params)


class OrderRefunds(RefundsBase, ResourceCreateMixin, ResourceListMixin):
    """Resource handler for the `/orders/:order_id:/refunds` endpoint."""

    _order: Order

    def __init__(self, client: "Client", order: Order) -> None:
        super().__init__(client)
        self._order = order

    def get_resource_path(self) -> str:
        return f"orders/{self._order.id}/refunds"

    def create(self, data: Optional[Dict[str, Any]] = None, **params: Any) -> Refund:
        """Create a refund for the order. When no data arg is given, a refund for all order lines is assumed."""
        if not data:
            data = {"lines": []}
        return super().create(data, **params)


class SettlementRefunds(RefundsBase, ResourceListMixin):
    """ResourceHandler for the `/settlements/:settlement_id:/refunds` endpoint."""

    _settlement: "Settlement"

    def __init__(self, client: "Client", settlement: "Settlement"):
        super().__init__(client)
        self._settlement = settlement

    def get_resource_path(self) -> str:
        return f"settlements/{self._settlement.id}/refunds"


class ProfileRefunds(RefundsBase):
    """
    Resource handler for the `/refunds?profileId=:profile_id:` endpoint.

    This is separate from the `Refunds` resource handler to make it easier to inject the profileId.
    """

    _profile: Profile

    def __init__(self, client: "Client", profile: Profile) -> None:
        self._profile = profile
        super().__init__(client)

    def list(self, **params: Any) -> ObjectList:
        # Set the profileId in the query params
        params.update({"profileId": self._profile.id})
        return Refunds(self.client).list(**params)



import re
from typing import Any

from ..error import IdentifierError
from ..objects.permission import Permission
from .base import ResourceGetMixin, ResourceListMixin

__all__ = [
    "Permissions",
]


class Permissions(ResourceGetMixin, ResourceListMixin):
    """Resource handler for the `/permissions` endpoint."""

    def get_resource_object(self, result: dict) -> Permission:
        return Permission(result, self.client)

    @staticmethod
    def validate_permission_id(permission_id: str) -> None:
        if not permission_id or not bool(re.match(r"^[a-z]+\.[a-z]+$", permission_id)):
            raise IdentifierError(f"Invalid permission ID: '{permission_id}'. Does not match ^[a-z]+.[a-z]+$")

    def get(self, resource_id: str, **params: Any) -> Permission:
        self.validate_resource_id(resource_id)
        return super().get(resource_id, **params)



from typing import Any

from ..objects.invoice import Invoice
from .base import ResourceGetMixin, ResourceListMixin

__all__ = [
    "Invoices",
]


class Invoices(ResourceGetMixin, ResourceListMixin):
    """Resource handler for the `/invoices` endpoint."""

    RESOURCE_ID_PREFIX: str = "inv_"

    def get_resource_object(self, result: dict) -> Invoice:
        return Invoice(result, self.client)

    def get(self, resource_id: str, **params: Any) -> Invoice:
        self.validate_resource_id(resource_id, "invoice ID")
        return super().get(resource_id, **params)



from typing import TYPE_CHECKING, Any

from ..objects.customer import Customer
from ..objects.mandate import Mandate
from .base import ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin

if TYPE_CHECKING:
    from ..client import Client


__all__ = [
    "CustomerMandates",
]


class CustomerMandates(ResourceCreateMixin, ResourceDeleteMixin, ResourceGetMixin, ResourceListMixin):
    """Resource handler for the `/customers/:customer_id:/mandates` endpoint."""

    RESOURCE_ID_PREFIX = "mdt_"

    _customer: Customer

    def __init__(self, client: "Client", customer: Customer) -> None:
        self._customer = customer
        super().__init__(client)

    def get_resource_path(self) -> str:
        return f"customers/{self._customer.id}/mandates"

    def get_resource_object(self, result: dict) -> Mandate:
        return Mandate(result, self.client)

    def get(self, resource_id: str, **params: Any) -> Mandate:
        self.validate_resource_id(resource_id, "mandate ID")
        return super().get(resource_id, **params)

    def delete(self, resource_id: str, **params: Any) -> dict:
        self.validate_resource_id(resource_id, "mandate ID")
        return super().delete(resource_id, **params)



from typing import Any, Dict

from ..error import IdentifierError
from ..objects.onboarding import Onboarding as OnboardingObject
from .base import ResourceGetMixin

__all__ = [
    "Onboarding",
]


class Onboarding(ResourceGetMixin):
    """Resource handler for the `/onboarding` endpoint."""

    def get_resource_object(self, result: dict) -> OnboardingObject:
        return OnboardingObject(result, self.client)

    def get(self, resource_id: str, **params: Any) -> OnboardingObject:
        if resource_id != "me":
            raise IdentifierError(f"Invalid onboarding ID: '{resource_id}'. The onboarding ID should be 'me'.")
        return super().get(resource_id, **params)

    def create(self, data: Dict[str, Any], **params: Any) -> OnboardingObject:
        resource_path = self.get_resource_path()
        path = f"{resource_path}/me"
        result = self.perform_api_call(self.REST_CREATE, path, data, params)
        return self.get_resource_object(result)



from .base import ObjectBase


class Customer(ObjectBase):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources import Customers

        return Customers(client)

    @property
    def id(self):
        return self._get_property("id")

    @property
    def name(self):
        return self._get_property("name")

    @property
    def email(self):
        return self._get_property("email")

    @property
    def locale(self):
        return self._get_property("locale")

    @property
    def metadata(self):
        return self._get_property("metadata")

    @property
    def mode(self):
        return self._get_property("mode")

    @property
    def resource(self):
        return self._get_property("resource")

    @property
    def created_at(self):
        return self._get_property("createdAt")

    @property
    def subscriptions(self):
        from ..resources import CustomerSubscriptions

        return CustomerSubscriptions(self.client, self)

    @property
    def mandates(self):
        from ..resources import CustomerMandates

        return CustomerMandates(self.client, self)

    @property
    def payments(self):
        from ..resources import CustomerPayments

        return CustomerPayments(self.client, self)
